/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.service;

import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.resp.team.TeamMember;
import lombok.RequiredArgsConstructor;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class TelegramMessageService {
    private final RabbitTemplate rabbitTemplate;

    public void sendMessage(String botToken, Long targetId, String message) {
        if (!containsHTMLCharacters(message)) {
            message = escapeMarkdown(message);
        }
        String fullMessage = String.format("%s|%s|%s", botToken, targetId, message);
        this.rabbitTemplate.convertAndSend("message_queue", fullMessage);
    }

    private static String escapeMarkdown(String text) {

        List<Character> escapeChars = Arrays
                .asList('_', '[', ']', '(', ')', '~', '>', '#', '+', '-', '=', '|', '{', '}', '.', '!');
        for (Character charItem : escapeChars) {
            text = text.replace(charItem.toString(), "\\" + charItem);
        }
        return text;
    }

    private static boolean containsHTMLCharacters(String text) {
        List<String> htmlTags = Arrays.asList("<b>", "<i>", "<a>", "<code>", "<pre>");
        if (htmlTags.stream().anyMatch(text::contains)) {
            return true;
        }
        return false;
    }

    public String buildRegistrationMessage(String accountName,
                                           List<TeamMember> accountMembers,
                                           TeamAccount teamAccount) {
        String memberNames = accountMembers.stream().map(TeamMember::getName).collect(Collectors.joining(", "));
        return String.format("👏 %s 注册: %d 用户: `%s` 总数:*%d*", accountName, accountMembers
                .size(), memberNames, teamAccount.getSubMemberNum());
    }

    public String buildDepositMessage(String agentName, int newDepositNum, String depositResults, int firstDepositNum) {
        return String.format("🎉 %s 首存: *%d* %s 总数: *%d*", agentName, newDepositNum, depositResults, firstDepositNum);
    }

    public String buildDepositResultsMessage(String name, BigDecimal scoreAmount) {
        return String.format("用户: `%s`, 首存金额: *%s*\n", name, scoreAmount);
    }
}
