package com.zayac.admin.utils;

import java.util.List;

public class TableFormatter {

    public static String formatMarkdownTable(List<String[]> rows) {
        int[] colWidths = calculateColumnWidths(rows);
        StringBuilder table = new StringBuilder("```\n");
        for (String[] row : rows) {
            table.append(formatRow(row, colWidths)).append("\n");
        }
        table.append("```");
        return table.toString();
    }

    public static String formatTable(List<String[]> rows) {
        int[] colWidths = calculateColumnWidths(rows);
        StringBuilder table = new StringBuilder();
        for (String[] row : rows) {
            table.append(formatRow(row, colWidths)).append("\n");
        }
        return table.toString();
    }

    public static String formatTableAsHtml(List<String[]> rows) {
        int[] colWidths = calculateColumnWidths(rows);
        StringBuilder table = new StringBuilder("<pre>\n");
        for (String[] row : rows) {
            table.append(formatRow(row, colWidths)).append("\n");
        }
        table.append("</pre>");
        return table.toString();
    }

    private static int[] calculateColumnWidths(List<String[]> rows) {
        int[] colWidths = new int[rows.get(0).length];
        for (String[] row : rows) {
            for (int i = 0; i < row.length; i++) {
                colWidths[i] = Math.max(colWidths[i], getStringWidth(row[i]));
            }
        }
        return colWidths;
    }

    private static String formatRow(String[] row, int[] colWidths) {
        StringBuilder formattedRow = new StringBuilder();
        for (int i = 0; i < row.length; i++) {
            formattedRow.append(padRight(row[i], colWidths[i])).append(" ");
        }
        return formattedRow.toString().trim();
    }

    private static int getStringWidth(String s) {
        return s.codePoints().map(ch -> ch > 0xFF ? 2 : 1).sum();
    }

    private static String padRight(String s, int n) {
        int width = getStringWidth(s);
        int padding = n - width;
        return s + " ".repeat(Math.max(0, padding));
    }
}
